#-------------------------------------------------------------------------------
# Name:        dataVisualizer
# Purpose:
#
# Author:      dccharacter
#
# Created:     23.03.2012
# Copyright:   (c) dccharacter 2012
# Licence:     <your licence>
#-------------------------------------------------------------------------------
#!/usr/bin/env python

from tkinter import *
from tkinter.ttk import *
from robowidgets.serialPortGui import *
import threading
import re
import atexit
import random
from datetime import datetime
import time
import queue

class KalmanFilterSimple1D:
    def __init__(self, q, r, f = 1, h = 1):
        self.X0 = 0.0 #predicted state
        self.P0 = 0.0 #predicted covariance

        self.F = 0.0 #factor of real value to previous real value
        self.Q = 0.0 #measurement noise
        self.H = 0.0 #factor of measured value to real value
        self.R = 0.0 #environment noise

        self.Q, self.R, self.F, self.H = q, r, f, h

        self.State = 0.0
        self.Covariance = 0.0

    def SetState(self, state, covariance):
        self.State, self.Covariance = state, covariance

    def Correct(self, data):
        #time update - prediction
        self.X0 = self.F * self.State
        self.P0 = self.F * self.Covariance * self.F + self.Q

        #measurement update - correction
        K = self.H * self.P0 / (self.H * self.P0 * self.H + self.R)
        self.State = self.X0 + K * (data - self.H * self.X0)
        self.Covariance = (1 - K * self.H) * self.F


class dataVisualizer(LabelFrame):
    def __init__(self, master, comport , cnf={}, **kw):
        self.master = master
        self.comport = comport
        LabelFrame.__init__(self, *cnf, **kw)
        self.pack()
        self.settingsFrame = LabelFrame(self, text="Settings")
        self.settingsFrame.pack()

        self.messageVar = StringVar()
        Label(self.settingsFrame, text="Message format regexp:").grid(row=0, column=0)
        self.messagePattern = Entry(self.settingsFrame, width = 40, text = 234234, textvariable = self.messageVar);
        self.messageVar.set(r'(-*\d+),(-*\d+),(-*\d+),(-*\d+),(-*\d+),(-*\d+)')
        self.messagePattern.grid(column=0, row=1)
        Button(self.settingsFrame, text = "Pause", command = self.pause).grid(column=1,row=0,rowspan=2)
        self.pauseFlag = TRUE

        self.t1var, self.t2var = StringVar(), StringVar()
        self.v1var, self.v2var = StringVar(), StringVar()
        self.dtvar, self.dvvar = StringVar(), StringVar()

        Label(self.settingsFrame, textvariable = self.t1var).grid(row=0, column=2)
        Label(self.settingsFrame, textvariable = self.v1var).grid(row=0, column=3)
        Label(self.settingsFrame, textvariable = self.t2var).grid(row=1, column=2)
        Label(self.settingsFrame, textvariable = self.v2var).grid(row=1, column=3)
        Label(self.settingsFrame, textvariable = self.dtvar).grid(row=2, column=2)
        Label(self.settingsFrame, textvariable = self.dvvar).grid(row=2, column=3)

        self.canvWidth, self.canvHeight = 1000, 700
        self.density = 4 ##width of pixel - the bigger, the wider graph
        self.numOfDots = self.canvWidth//self.density
        self.numOfGraphs = 6
        self.timings = []
        self.graphData = []
        self.graphLines = []
        self.zeroTiming = datetime.now()
        self.colors = ['red', 'blue', 'green', 'orange', 'violet', 'black', 'cyan']
        self.prevTuple = (0,)*10
        self.timingIndex = 0

        self.canv = Canvas(self, width = self.canvWidth, height = self.canvHeight,
            relief=GROOVE, background='snow',bd=5)
        self.canv.pack()
        self.canv.bind("<Button-1>",self.mouseClick)
        self.canv.bind("<Button-3>",self.mouseDoubleClick)

        self.queue = queue.Queue()
        self.thread = threading.Thread(target = self.workerThread)
        self.thread.start()

        self.kalman = KalmanFilterSimple1D(10, 20.3, 1, 1)
        self.kalman.SetState(0, 0.1)

        self.drawGraph()
        self.drawMesh()
        self.periodicCall()

    def mouseClick (self, event):
        try:
            self.t1var.set('t1 = '+str(self.timings[(self.canv.winfo_reqwidth() - int(event.x))//self.density]))
        except IndexError: pass
        self.v1var.set('v1 = '+str(self.canv.winfo_reqheight()//2 - int(event.y)))
        self.calculate()

    def mouseDoubleClick (self, event):
        try:
            self.t2var.set('t2 = '+str(self.timings[(self.canv.winfo_reqwidth() - int(event.x))//self.density]))
        except IndexError: pass
        self.v2var.set('v2 = '+str(self.canv.winfo_reqheight()//2 - int(event.y)))
        self.calculate()

    def calculate(self):
        pass

    def pause(self):
        self.pauseFlag = ~self.pauseFlag

    def drawGraph(self):
        for graphNum in range(0, self.numOfGraphs):
            self.graphLines.append([])
            self.graphData.append([0,]*self.numOfDots)
            for iii in range(0,self.numOfDots):
                self.graphLines[graphNum].append(
                    self.canv.create_line(0,0,0,0,fill=self.colors[graphNum],
                    width=1.2, tags=('graphLines', 'graph'+str(graphNum)))
                    )


    def updateGraph3(self):##KALMAN
        while not self.queue.empty():
            iTuple = self.queue.get()
            self.canv.move('graphLines', -self.density,0)
            try: self.graphData[0].append(int(iTuple[0]))
            except IndexError:
                self.graphData[0].append(0)
            self.graphData[0].pop(0)
            for iii in range(0,len(self.graphData[0])):
                self.kalman.Correct(int(self.graphData[0][iii]))
                self.graphData[1].pop(0)
                self.graphData[1].append(self.kalman.State)
            self.graphData[2].append(sum(self.graphData[0][-2:])//2) ##running average




            for lineNum in range(0, len(self.graphLines)):
                self.graphLines[lineNum].append(self.graphLines[lineNum].pop(0))
                self.canv.coords(self.graphLines[lineNum][-1],
                    self.canv.winfo_reqwidth()-self.density,
                    int(int(self.graphData[lineNum][-2])+int(self.canv.winfo_reqheight()//2)),
                    self.canv.winfo_reqwidth(),
                    int(int(self.graphData[lineNum][-1])+int(self.canv.winfo_reqheight()//2))
                    )

    def updateGraph2(self):##reuses same lines for drawing, remembers lines data
        while not self.queue.empty():
            iTuple = self.queue.get()
            self.canv.move('graphLines', -self.density,0)
            for graphNum in range(0, self.numOfGraphs):
                try: self.graphData[graphNum].append(int(iTuple[graphNum]))
                except IndexError:
                    self.graphData[graphNum].append(0)
                self.graphData[graphNum].pop(0)
                self.graphLines[graphNum].append(self.graphLines[graphNum].pop(0))
                self.canv.coords(self.graphLines[graphNum][-1],
                    self.canv.winfo_reqwidth()-self.density,
                    self.canv.winfo_reqheight()//2 - self.graphData[graphNum][-2],
                    self.canv.winfo_reqwidth(),
                    self.canv.winfo_reqheight()//2- self.graphData[graphNum][-1]
                    )

    def updateGraph(self): ##creates new items, does not remember line data
        while not self.queue.empty():
            self.timingIndex = self.timingIndex + 1
            self.canv.move('graphLines', -self.density, 0)
            iTuple = self.queue.get()
            for iii in range(0, len(iTuple)):
                yyy = int(iTuple[iii])+self.canv.winfo_reqheight()//2
                if yyy < 0: yyy = 0
                if yyy > self.canv.winfo_reqheight(): yyy = self.canv.winfo_reqheight()
                prev_yyy = int(self.prevTuple[iii])+self.canv.winfo_reqheight()//2
                if prev_yyy < 0: prev_yyy = 0
                if prev_yyy > self.canv.winfo_reqheight(): prev_yyy = self.canv.winfo_reqheight()
                self.canv.create_line(
                    self.canv.winfo_reqwidth()-self.density, prev_yyy,
                    self.canv.winfo_reqwidth(), yyy,
                    width = 1.4, fill = self.colors[iii], tags=('graphLines','graph'+str(iii)))
            self.prevTuple = iTuple

            self.canv.addtag_overlapping('todelete',-1,-1,-3,self.canv.winfo_reqheight()+1)
            self.canv.dtag('preserve','todelete')
            self.canv.delete('todelete')


    def drawMesh(self):
        self.zeroLine = self.canv.create_line(0, self.canv.winfo_reqheight()//2, self.canv.winfo_reqwidth(),
            self.canv.winfo_reqheight()//2, fill="black", width = 1, tags=('ontop','preserve'))

        self.timeText = self.canv.create_text(-50,-50,
                text = "t=",
                anchor = W, tags=('ontop','textIndicator','preserve'))
        self.valText = self.canv.create_text(-50,-50,
                text = "value=",
                anchor = W, tags=('ontop','textIndicator','preserve'))
        self.tipRect = self.canv.create_rectangle(0,0,0,0, outline = "black",
            fill = "lightyellow", tags=('ontop','preserve','textbox'))
        self.legendRec = self.canv.create_rectangle(0,0,0,0, outline = "black",
            fill = "lightyellow", tags=('ontop','preserve','legendbox'))
        for graphNumber in range(0,len(self.graphLines)):
            self.canv.create_rectangle(self.canv.winfo_reqwidth()-120, graphNumber*30+30,
                self.canv.winfo_reqwidth()-100, graphNumber*30+50, tags = ('ontop','legend','legendonoff', 'preserve'),
                outline = "black", fill = self.colors[graphNumber])
            self.canv.create_text(self.canv.winfo_reqwidth()-90, graphNumber*30+40,
                tags = ('ontop','legend', 'preserve'), anchor=W,text='line '+str(graphNumber),
                fill = self.colors[graphNumber])



        self.canv.create_line(10, 10, 100, 100, tags=('ontop', "verPntr",'preserve'),
            fill="blue", dash = [4, 1, 2, 1], width = 1)
        self.canv.create_line(-10, -10, -10, -10, tags=('ontop', "horPntr",'preserve'),
            fill="blue", dash = [4, 1, 2, 1], width = 1)
        self.canv.lift('ontop')
        self.canv.lift('textIndicator')

    def updateMesh(self):
        aaa, bbb = 0, 0
        mouseX = self.canv.winfo_pointerx() - self.canv.winfo_rootx()
        mouseY = self.canv.winfo_pointery() - self.canv.winfo_rooty()
        self.canv.coords("verPntr", (mouseX, 0, mouseX, self.canv.winfo_reqheight()))
        self.canv.coords("horPntr", (0, mouseY, self.canv.winfo_reqwidth(), mouseY))
        if mouseY < 50: aaa = -1
        else: aaa = 1
        if mouseX > self.canv.winfo_reqwidth()*0.8: bbb = -13
        else: bbb = 1
        self.canv.coords(self.timeText, mouseX+bbb*10, mouseY-aaa*30)
        self.canv.coords(self.valText, mouseX+bbb*10, mouseY-aaa*15)
        self.canv.itemconfig(self.valText, text = 'value = '+str(self.canv.winfo_reqheight()//2 - mouseY))
        try:
            self.canv.itemconfig(self.timeText, text = 't = '+str(self.timings[
                (self.canv.winfo_reqwidth() - mouseX)//self.density
                ]))
        except IndexError: pass
        self.canv.coords('textbox', self.canv.bbox('textIndicator'))
        ##self.canv.coords('legendbox', self.canv.bbox('legend'))
        self.canv.lift('preserve')


    def periodicCall(self):
        self.updateGraph2()
        self.updateMesh()
        self.after(150, func=self.periodicCall)


    def workerThread(self):

        while (1):
            try:
                if self.comport.isOpen() and (self.pauseFlag == TRUE):
                    comLine = self.comport.readline()
                    if len(self.timings) == self.numOfDots:
                        self.timings.pop(-1)
                    parsedLine = re.search(self.messagePattern.get(), str(comLine))
                    if parsedLine:
                        self.timings.insert(0, datetime.now() - self.zeroTiming)
                        self.queue.put(parsedLine.groups())
                else:
                    self.comport.flush();
                    time.sleep(1)
            except TclError:
                self.thread._stop()


def main():
    root = Tk()
    mainWindow = Frame(root)
    mainWindow.pack()
    port = comPortWidget(mainWindow)
    port.pack()
    dv = dataVisualizer(mainWindow, port)
    dv.pack()
    root.mainloop()

if __name__ == '__main__':
    main()
