#-------------------------------------------------------------------------------
# Name:        dccharacterBatteyModule
# Purpose:
#
# Author:      dccharacter
#
# Created:     11.03.2012
# Copyright:   (c) dccharacter 2012
# Licence:     <your licence>
#-------------------------------------------------------------------------------
#!/usr/bin/env python

from tkinter import *
from tkinter.ttk import *

class dccharacterBatteryWidget(LabelFrame):

    def __init__(self, master, sendToSerialCommand, cnf={}, **kw):
        self.statusVinVar, self.statusVoutVar, self.statusIloadVar = StringVar(), StringVar(), StringVar()
        self.enableVoutVar, self.enableLoadVar, self.enableSelfTestVar = IntVar(), IntVar(), IntVar()
        self.sendCommandToSerial = sendToSerialCommand
        LabelFrame.__init__(self, master, *cnf, **kw)

        self.fillVoltageFrame()
        self.fillEnableFrame()
        self.fillStatusFrame()
        self.fillPIDSettingsFrame()

    def fillVoltageFrame(self):
        self.frameLabelVoltageSettings = LabelFrame(self, text = "Voltage settings", labelanchor="nw")
        self.frameLabelVoltageSettings.grid(row=0, column=0)

        Label(self.frameLabelVoltageSettings, text="Output Voltage:").grid(row=0,column=0)
        self.entryOutputVoltage = Spinbox(self.frameLabelVoltageSettings, from_=0, to=1023, width=4).grid(row=0, column=1)
        Label(self.frameLabelVoltageSettings, text="Warning threshold:").grid(row=1,column=0)
        self.entryWarningThreshold = Spinbox(self.frameLabelVoltageSettings, from_=0, to=1023, width=4).grid(row=1, column=1)
        Label(self.frameLabelVoltageSettings, text="Cut off threshold:").grid(row=2,column=0)
        self.entryCutOffThreshold = Spinbox(self.frameLabelVoltageSettings, from_=0, to=1023, width=4).grid(row=2, column=1)
        Label(self.frameLabelVoltageSettings, text="System halt threshold:").grid(row=3,column=0)
        self.entrySystemHalt = Spinbox(self.frameLabelVoltageSettings, from_=0, to=1023, width=4).grid(row=3, column=1)
        Label(self.frameLabelVoltageSettings, text="Hysteresis:").grid(row=4,column=0)
        self.entryHysteresis = Spinbox(self.frameLabelVoltageSettings, from_=0, to=1023, width=4).grid(row=4, column=1)

        self.buttonUpdateVoltageSettings = Button(self.frameLabelVoltageSettings, text="Update Voltage Settings").grid(row=5, column=0, columnspan=2)

    def fillEnableFrame(self):
        self.frameLabelEnable = LabelFrame(self, text = "Enable", labelanchor="nw")
        self.frameLabelEnable.grid(row=1, column=0)

        self.checkbuttonEnableOutput = Checkbutton(self.frameLabelEnable, command=self.sendEnableOutputAndLoad,
            variable = self.enableVoutVar, text="Enable Output:")
        self.checkbuttonEnableOutput.pack()
        self.checkbuttonEnableLoad = Checkbutton(self.frameLabelEnable, command=self.sendEnableOutputAndLoad,
            variable = self.enableLoadVar, text="Enable Load:")
        self.checkbuttonEnableLoad.pack()
        self.checkbuttonEnableLedTest = Checkbutton(self.frameLabelEnable, command=self.sendEnableLedTest,
            variable = self.enableSelfTestVar, text="Enable Led Test:")
        self.checkbuttonEnableLedTest.pack()

    def fillStatusFrame(self):
        self.frameLabelDeviceStatus = LabelFrame(self, text = "Device status", labelanchor="nw")
        self.frameLabelDeviceStatus.grid(row=1, column=1)

        Label(self.frameLabelDeviceStatus, text="Vin:").grid(row=0,column=0)
        Label(self.frameLabelDeviceStatus, text="Vout:").grid(row=1,column=0)
        Label(self.frameLabelDeviceStatus, text="Current:").grid(row=2,column=0)
        self.labelStatusVin = Label(self.frameLabelDeviceStatus, textvariable=self.statusVinVar,
            text="Waiting for update...")
        self.labelStatusVin.grid(row=0,column=1)
        self.labelStatusVout = Label(self.frameLabelDeviceStatus, textvariable=self.statusVoutVar,
            text="Waiting for update...")
        self.labelStatusVout.grid(row=1,column=1)
        self.labelStatusCurrent = Label(self.frameLabelDeviceStatus, textvariable=self.statusIloadVar,
            text="Waiting for update...")
        self.labelStatusCurrent.grid(row=2,column=1)


    def fillPIDSettingsFrame(self):
        KpVar, KiVar, KdVar = StringVar(), StringVar(), StringVar()
        self.frameLabelPIDSettings = LabelFrame(self, text = "PID settings", labelanchor="nw")
        self.frameLabelPIDSettings.grid(row=0, column=1)

        Label(self.frameLabelPIDSettings, text="Kp:").grid(row=0,column=0)
        self.scalePidKp = Scale(self.frameLabelPIDSettings,from_=0, to=100, orient=HORIZONTAL)
        self.scalePidKp.grid(row=0,column=1)
        self.labelPidKpValue = Label(self.frameLabelPIDSettings,textvariable=KpVar, text="val").grid(row=0,column=2)
        ##self.scalePidKp.bind


        Label(self.frameLabelPIDSettings, text="Ki:").grid(row=1,column=0)
        self.scalePidKi = Scale(self.frameLabelPIDSettings, from_=0, to=100, orient=HORIZONTAL).grid(row=1,column=1)
        self.labelPidKiValue = Label(self.frameLabelPIDSettings, text="val").grid(row=1,column=2)
        Label(self.frameLabelPIDSettings, text="Kd:").grid(row=2,column=0)
        self.scalePidKd = Scale(self.frameLabelPIDSettings, from_=0, to=100, orient=HORIZONTAL).grid(row=2,column=1)
        self.labelPidKdValue = Label(self.frameLabelPIDSettings, text="val").grid(row=2,column=2)

        self.buttonUpdatePidSettings = Button(self.frameLabelPIDSettings, text="Update Pid Settings").grid(row=3, column=1)

    def sendEnableLedTest(self):
        pass

    def sendEnableOutputAndLoad(self):
        outputByte = self.enableVoutVar.get()
        loadByte = self.enableLoadVar.get()
        self.sendCommandToSerial(b's\x0E'+bytes([outputByte, loadByte])+b'p')

def main():
    import RouterPower
    RouterPower.main()

if __name__ == '__main__':
    main()
